<?xml version="1.0"?>

<!-- 
This is the XSLT file that defines all sections of the final report.

This file includes the following four files to generate the report:

1. executive_summary.xsl
2. table_of_contents.xsl
3. technical_report.xsl
4. footer.xsl

-->

<xsl:stylesheet version="2.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:meet="java:com.ibm.extremeblue.meet.xutils.MeetXUtils">

<xsl:include href="executive_summary.xsl" />
<xsl:include href="../table_of_contents.xsl" />
<xsl:include href="technical_report.xsl" />
<xsl:include href="../footer.xsl" />

<xsl:template match="/">
	<html>
		<style>
			<xsl:value-of select="meet:loadCSS()"/>
		</style>
		<script>
			<xsl:value-of select="meet:loadJavascript()"/>
		</script>
		<head>
			<title>MEET DB2 
			<xsl:choose>
				<xsl:when test="exists(report/executeinfo/version/query)">
					<xsl:value-of select="report/executeinfo/version/query" />
				</xsl:when>
				<xsl:otherwise>
					9.7 FP1
				</xsl:otherwise>
			</xsl:choose>
			 Report
			<xsl:choose>
				<xsl:when test="exists(report/executeinfo/datetime)">
					<xsl:variable name="reportdatetime" select="report/executeinfo/datetime" />
					<xsl:value-of select="format-dateTime($reportdatetime, '[Y0001]/[M01]/[D01]') " />
				</xsl:when>
				<xsl:otherwise>
					<xsl:variable name="reportdatetime" select="current-dateTime()" />
					<xsl:value-of select="format-dateTime($reportdatetime, '[Y0001]/[M01]/[D01]') " />
				</xsl:otherwise>
			</xsl:choose>
			  <xsl:if test="report/executeinfo/internalexternal = 'internal'">
			  (IBM Confidential)
			  </xsl:if>
			  <xsl:if test="report/executeinfo/internalexternal = 'external' or not(exists(report/executeinfo/internalexternal))">
			  (Internal Use Only)
			  </xsl:if>
			  </title>
		</head>
		<body>
			<h1>MEET DB2 
			<xsl:choose>
				<xsl:when test="exists(report/executeinfo/version/query)">
					<xsl:value-of select="report/executeinfo/version/query" />
				</xsl:when>
				<xsl:otherwise>
					9.7 FP1
				</xsl:otherwise>
			</xsl:choose>
			 Report
			<xsl:choose>
				<xsl:when test="exists(report/executeinfo/datetime)">
					<xsl:variable name="reportdatetime" select="report/executeinfo/datetime" />
					<xsl:value-of select="format-dateTime($reportdatetime, '[Y0001]/[M01]/[D01]') " />
				</xsl:when>
				<xsl:otherwise>
					<xsl:variable name="reportdatetime" select="current-dateTime()" />
					<xsl:value-of select="format-dateTime($reportdatetime, '[Y0001]/[M01]/[D01]') " />
				</xsl:otherwise>
			</xsl:choose>
			  <xsl:if test="report/executeinfo/internalexternal = 'internal'">
			  (IBM Confidential)
			  </xsl:if>
			  <xsl:if test="report/executeinfo/internalexternal = 'external' or not(exists(report/executeinfo/internalexternal))">
			  (Internal Use Only)
			  </xsl:if>
			</h1>
			<p><b>Migration Enablement Evaluation Tool for DB2</b><br/>
				Send comments to
				<a>	<xsl:attribute name="href">
					<xsl:text>mailto:</xsl:text>
					<xsl:value-of select="meet:getConfig('COMMENTS_ADDRESS')"/>
					<xsl:text>?subject=Comments%20on%20MEET%20DB2%20(v</xsl:text>
					<xsl:choose>
						<xsl:when test="exists(report/executeinfo/version/meet)">
							<xsl:value-of select="report/executeinfo/version/meet"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:text>0.51</xsl:text>
						</xsl:otherwise>
					</xsl:choose>
					<xsl:text>)</xsl:text>
					</xsl:attribute>
					<xsl:value-of select="meet:getConfig('COMMENTS_ADDRESS')"/>
				</a>
			</p>
			
			<xsl:apply-templates select="report" mode="executive_summary"/>
<!-- 			<xsl:call-template name="table_of_contents"/>	take out for beta 3 -->
			<xsl:apply-templates select="report" mode="technical_report"/>
			<xsl:call-template name="footer"/>
		</body>
	</html>
</xsl:template>

</xsl:stylesheet>
