<?xml version="1.0" encoding="ISO-8859-1"?>

<xsl:stylesheet version="2.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:meet="java:com.ibm.extremeblue.meet.xutils.MeetXUtils">

<xsl:template match="report" mode="technical_report">

	<a name="technical_summary"/>
	<h2>Technical Summary</h2>

	<p>
		<b>Report generated on: </b>
		<xsl:choose>
			<xsl:when test="exists(executeinfo/datetime)">
				<xsl:variable name="reportdatetime" select="executeinfo/datetime" />
				<xsl:value-of select="format-dateTime($reportdatetime, '[Y0001]/[M01]/[D01]')" />
				<xsl:text> at </xsl:text>
				<xsl:value-of select="format-dateTime($reportdatetime, '[h]:[m01] [Pn]')" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="reportdatetime" select="current-dateTime()" />
				<xsl:value-of select="format-dateTime($reportdatetime, '[Y0001]/[M01]/[D01]')" />
				<xsl:text> at </xsl:text>
				<xsl:value-of select="format-dateTime($reportdatetime, '[h]:[m01] [Pn]')" />
			</xsl:otherwise>
		</xsl:choose>
	</p>
	<p>
		<b>Source 
	    <xsl:value-of select="sources/source/type"/>: <xsl:text>&#160;</xsl:text>
	    </b>
	    <xsl:value-of select="sources/source/name"/>
	</p>
<!-- <ul>
		<xsl:for-each select="sources/source">
			<li><b><xsl:value-of select="type"/>:</b><xsl:text>&#160;</xsl:text><xsl:value-of select="name"/></li>
		</xsl:for-each>
	</ul>
-->	
	<h3>PL/SQL Summary</h3>
	<table id="plsql_summary">
		<thead>
			<tr>
				<th>Object Type</th>
				<th>Total Number</th>
				<th>Number That Require Attention</th>
				<th>Percent That Require Attention</th>
				<!-- removed effort estimates -->
				<!-- <th class="effort">Effort <br/>(person hours)</th> -->
			</tr>
		</thead>
		<tbody>
			<!-- first pull out the detail elements, so that they can be sorted -->
			<xsl:for-each select="summary/item">
				<xsl:sort select="percent_incompatible" data-type="number" order="ascending"/>
				<xsl:sort select="number" data-type="number" order="ascending"/>
				<xsl:if test="object_type != 'Total Objects' and object_type != 'Statements'">
					<tr class="detail">
						<td><xsl:value-of select="object_type"/></td>
						<td><xsl:value-of select="number"/></td>
						<td><xsl:value-of select="number_incompatible"/></td>
						<td><xsl:value-of select="percent_incompatible"/> %</td>
						<!-- removed effort estimates -->
						<!-- <td class="effort"><xsl:apply-templates select="effort"/></td> -->
					</tr>
				</xsl:if>
			</xsl:for-each>
			<!-- then, pull out the summary elements, so they'll be at the bottom -->
			<xsl:for-each select="summary/item">
				<xsl:if test="object_type = 'Total Objects' or object_type = 'Statements'">
					<tr class="summary">
						<td><xsl:value-of select="object_type"/></td>
						<td><xsl:value-of select="number"/></td>
						<td><xsl:value-of select="number_incompatible"/></td>
						<td><xsl:value-of select="percent_incompatible"/> %</td>
						<!-- removed effort estimates -->
						<!-- <td class="effort"><xsl:apply-templates select="effort"/></td> -->
					</tr>
				</xsl:if>
			</xsl:for-each>
		</tbody>
	</table>
	
	<h3>DDL Summary</h3>
	<p>
		The DDL reports assume that the IBM Data Movement Tool will be used to migrate DDL
		to DB2. 
	</p>
	<table id="ddl_summary">
		<thead>
			<tr>
				<th>Statement Type</th>
				<th>Total Number</th>
				<th>Number That Require Attention</th>
				<th>Percent That Require Attention</th>
				<!-- removed effort estimates -->
				<!-- <th class="effort">Effort <br/>(person hours)</th> -->
			</tr>
		</thead>
		<tbody>
		<!-- first pull out the detail elements, so that they can be sorted -->
			<xsl:for-each select="ddlsummary/item">
				<xsl:sort select="percent_incompatible" data-type="number" order="ascending"/>
				<xsl:sort select="number" data-type="number" order="ascending"/>
				<xsl:if test="object_type != 'Total Objects' and object_type != 'Statements'">
					<tr class="detail">
						<td><xsl:value-of select="object_type"/></td>
						<td><xsl:value-of select="number"/></td>
						<td><xsl:value-of select="number_incompatible"/></td>
						<td><xsl:value-of select="percent_incompatible"/> %</td>
						<!-- removed effort estimates -->
						<!-- <td class="effort"><xsl:apply-templates select="effort"/></td> -->
					</tr>
				</xsl:if>
			</xsl:for-each>
			<!-- then, pull out the summary elements, so they'll be at the bottom -->
			<xsl:for-each select="ddlsummary/item">
<!--				<xsl:if test="object_type = 'Total Objects' or object_type = 'Statements'"> -->
				<xsl:if test="object_type = 'Statements'">
					<tr class="summary">
						<td><xsl:value-of select="object_type"/></td>
						<td><xsl:value-of select="number"/></td>
						<td><xsl:value-of select="number_incompatible"/></td>
						<td><xsl:value-of select="percent_incompatible"/> %</td>
						<!-- removed effort estimates -->
						<!-- <td class="effort"><xsl:apply-templates select="effort"/></td> -->
					</tr>
				</xsl:if>
			</xsl:for-each>
		</tbody>
	</table>
	
	<p>
		<a class="show" id="showTechnicalReport" href="javascript:showHide('technical_details');showObject('hideTechnicalReport');hideObject('showTechnicalReport');">Click here to show the detailed technical report</a>
		<a class="hide" id="hideTechnicalReport" href="javascript:showHide('technical_details');showObject('showTechnicalReport');hideObject('hideTechnicalReport');">Click here to hide the detailed technical report</a>
	</p>
	
	<div id="technical_details" class="hide">
			
		<!-- Generate PL/SQL Statistics -->
		<a name="plsql_statistics"/>
		<xsl:call-template name="statisticsTable">
			<xsl:with-param name="title">Procedural SQL Statistics</xsl:with-param>
			<xsl:with-param name="items" select="plsql/statistics"/>
		</xsl:call-template>

		<a name="plsql_grouped_by_features"/>
		<h3>Procedural SQL Grouped By Features (Count: <xsl:value-of select="count(plsql/features/feature[@informational='false'])"/>)</h3>
		<p class="displayMenu">
			<a href="javascript: displayDetailsInTable('detailedPlsqlTableByFeatures')">Show Details</a> | 
			<a href="javascript: hideDetailsInTable('detailedPlsqlTableByFeatures')">Hide Details</a>
		</p>
		<table id="detailedPlsqlTableByFeatures">
			<thead>
				<th class="feature">Feature</th>
				<th class="description">Description</th>
				<th class="count">#</th>
				<!-- removed effort estimates -->
				<!-- <th class="effort">Effort (person hours)</th> -->
			</thead>
			<xsl:for-each select="plsql/features/feature[@informational='false']">
				<tr class="summary">
					<xsl:attribute name="onClick">showHide("<xsl:value-of select="name" />")</xsl:attribute> 
					<td class="feature"><xsl:value-of select="name"/></td>
					<td class="description"><xsl:value-of select="description"/></td>
					<td class="count"><xsl:value-of select="count"/></td>
					<!-- removed effort estimates -->
					<!-- <td class="effort"><xsl:apply-templates select="effort"/></td> -->
				</tr>
				<tr class="detail_container"><td colspan="4" class="detail_container">
					<table class="hide">
					<xsl:attribute name="id">
						<xsl:value-of select="name"/>
					</xsl:attribute>
		
					<xsl:for-each select="instances/instance">
					<tr class="detail">
						<td class="lineno">
									Line <xsl:value-of select="lineno"/>
						</td>
						<!--
						<td class="package">Package: <xsl:value-of select="package"/></td>
						-->
						<td class="object"><xsl:apply-templates select="object"/></td>
						<td class="context"><xsl:value-of select="context"/></td>
					</tr>
					</xsl:for-each>
					
					<xsl:if test="not(solution='')">
						<tr class="detail">
							<td valign="top" class="solution">Solution:</td>
							<td class="solution" colspan="2"><xsl:value-of select="solution"/></td>
						</tr>
					</xsl:if>
					</table>
				</td></tr>
			</xsl:for-each>
		</table>

		<xsl:if test="count(plsql/features/feature[@informational='true']) > 0">
			<a name="plsql_informational_features"/>
			<h3>Procedural SQL Features - Informational (Count: <xsl:value-of select="count(plsql/features/feature[@informational='true'])"/>)</h3>
			<p>
				MEET DB2 is sometimes not able to evaluate if a statement is
				supported by DB2. These instances should be manually evaluated. 
			</p>
			<p class="displayMenu">
				<a href="javascript: displayDetailsInTable('detailedPlsqlTableInformationalFeatures')">Show Details</a> | 
				<a href="javascript: hideDetailsInTable('detailedPlsqlTableInformationalFeatures')">Hide Details</a>
			</p>
			<table id="detailedPlsqlTableInformationalFeatures">
				<thead>
					<th class="feature">Feature</th>
					<th class="description">Description</th>
					<th class="count">#</th>
					<!-- removed effort estimates -->
					<!-- <th class="effort">Effort (person hours)</th> -->
				</thead>
				<xsl:for-each select="plsql/features/feature[@informational='true']">
					<tr class="summary">
						<xsl:attribute name="onClick">showHide("<xsl:value-of select="name" />")</xsl:attribute> 
						<td class="feature"><xsl:value-of select="name"/></td>
						<td class="description"><xsl:value-of select="description"/></td>
						<td class="count"><xsl:value-of select="count"/></td>
						<!-- removed effort estimates -->
						<!-- <td class="effort"><xsl:apply-templates select="effort"/></td> -->
					</tr>
					<tr class="detail_container"><td colspan="4" class="detail_container">
						<table class="hide">
						<xsl:attribute name="id">
							<xsl:value-of select="name"/>
						</xsl:attribute>
			
						<xsl:for-each select="instances/instance">
						<tr class="detail">
							<td class="lineno">
										Line <xsl:value-of select="lineno"/>
							</td>
							<!--
							<td class="package">Package: <xsl:value-of select="package"/></td>
							-->
							<td class="object"><xsl:apply-templates select="object"/></td>
							<td class="context"><xsl:value-of select="context"/></td>
						</tr>
						</xsl:for-each>
						
						<xsl:if test="not(solution='')">
							<tr class="detail">
								<td valign="top" class="solution">Solution:</td>
								<td class="solution" colspan="2"><xsl:value-of select="solution"/></td>
							</tr>
						</xsl:if>
						</table>
					</td></tr>
				</xsl:for-each>
			</table>
		</xsl:if>

		
		<!-- removed for beta1 - note: had to comment out in several places due to internal comments -->
		<!--
		<a name="plsql_grouped_by_objects"/>
		<h3>Procedural SQL Grouped By Objects (Count: <xsl:value-of select="count(plsql/objects/object)"/>)</h3>
			<p class="displayMenu">
			<a href="javascript: displayDetailsInTable('detailedPlsqlTableByObjects')">Show Details</a> | 
			<a href="javascript: hideDetailsInTable('detailedPlsqlTableByObjects')">Hide Details</a>
		</p>
		<table id="detailedPlsqlTableByObjects">
			<thead>
				<th class="object">Object</th>
				<th class="count">#</th>
		-->
				<!-- removed effort estimates -->
				<!-- <th class="effort">Effort (person hours)</th> -->
		<!--
			</thead>
			<xsl:for-each select="plsql/objects/object">
				<xsl:sort select="object/@type"/>
				<xsl:sort select="object"/>
				<tr class="summary">
					<xsl:attribute name="onClick">showHide("<xsl:value-of select="object" />")</xsl:attribute> 
					<td class="object"><xsl:apply-templates select="object"/></td>
					<td class="count"><xsl:value-of select="count"/></td>
		-->
					<!-- removed effort estimates -->
					<!-- <td class="effort"><xsl:apply-templates select="effort"/></td> -->
		<!--
				</tr>
				<tr class="detail_container"><td colspan="5" class="detail_container">
				<table class="hide">
					<xsl:attribute name="id">
						<xsl:value-of select="object"/>
					</xsl:attribute>
								
					<xsl:for-each select="instances/instance">
						<xsl:call-template name="plsqlInstanceByOject">
							<xsl:with-param name="instance" select="."/>
						</xsl:call-template>
					</xsl:for-each>
				</table>
				</td></tr>
			</xsl:for-each>
		</table>
		-->

		<!-- Generate DDL Statistics -->
		<a name="ddl_statistics"/>
		<xsl:call-template name="statisticsTable">
			<xsl:with-param name="title">DDL Statistics</xsl:with-param>
			<xsl:with-param name="items" select="ddl/statistics"/>
			<xsl:with-param name="text">
				The statistics provided below for DDL objects are the sum of create, drop
				and alter for the object type.
			</xsl:with-param>
		</xsl:call-template>
		
		<a name="ddl_grouped_by_features"/>
		<h3>DDL Grouped By Features (Count: <xsl:value-of select="count(ddl/features/feature[@informational='false'])"/>)</h3>
		<p class="displayMenu">
			<a href="javascript: displayDetailsInTable('detailedDdlTableByFeatures')">Show Details</a> | 
			<a href="javascript: hideDetailsInTable('detailedDdlTableByFeatures')">Hide Details</a>
		</p>
		<table id="detailedDdlTableByFeatures">
			<thead>
				<th class="feature">Feature</th>
				<th class="description">Description</th>
				<th class="count">#</th>
				<!-- removed effort estimates -->
				<!-- <th class="effort">Effort (person hours)</th> -->
			</thead>
			<xsl:for-each select="ddl/features/feature[@informational='false']">
				<tr class="summary">
					<xsl:attribute name="onClick">showHide("<xsl:value-of select="name" />")</xsl:attribute> 
					<td class="feature"><xsl:value-of select="name"/></td>
					<td class="description"><xsl:value-of select="description"/></td>
					<td class="count"><xsl:value-of select="count"/></td>
					<!-- removed effort estimates -->
					<!-- <td class="effort"><xsl:apply-templates select="effort"/></td> -->
				</tr>
				<tr class="detail_container"><td colspan="4" class="detail_container">
					<table class="hide">
						<xsl:attribute name="id">
							<xsl:value-of select="name"/>
						</xsl:attribute>
			
						<xsl:for-each select="instances/instance">
							<tr class="detail">
								<!-- removed and replaced with items below for reading ddl from file instead of DB connection -->
								<!--
								<td class="location"><xsl:apply-templates select="location"/></td>
								<td class="object"><xsl:apply-templates select="object"/></td>
								-->
								<td class="lineno">
									Line <xsl:value-of select="lineno"/>
								</td>
								<td class="object"><xsl:apply-templates select="object"/></td>
								<td class="context"><xsl:value-of select="context"/></td>
							</tr>
						</xsl:for-each>
						<xsl:if test="not(solution='')">
							<tr class="detail">
								<td valign="top" class="solution">Solution:</td>
								<td class="solution" colspan="2"><xsl:value-of select="solution"/></td>
							</tr>
						</xsl:if>
					</table>
				</td></tr>
			</xsl:for-each>
		</table>
		

		<xsl:if test="count(ddl/features/feature[@informational='true']) > 0">
			<a name="ddl_informational_features"/>
			<h3>DDL Features - Informational (Count: <xsl:value-of select="count(ddl/features/feature[@informational='true'])"/>)</h3>
			<p>
				The Data Movement Tool will ignore some options for DDL where there is no 
				equivalent in DB2, most of these ignored options are reported as informational 
				as they do not require any action in order to migrate to DB2.
			</p>
			<p class="displayMenu">
				<a href="javascript: displayDetailsInTable('detailedDdlTableInformationalFeatures')">Show Details</a> | 
				<a href="javascript: hideDetailsInTable('detailedDdlTableInformationalFeatures')">Hide Details</a>
			</p>
			<table id="detailedDdlTableInformationalFeatures">
				<thead>
					<th class="feature">Feature</th>
					<th class="description">Description</th>
					<th class="count">#</th>
					<!-- removed effort estimates -->
					<!-- <th class="effort">Effort (person hours)</th> -->
				</thead>
				<xsl:for-each select="ddl/features/feature[@informational='true']">
					<tr class="summary">
						<xsl:attribute name="onClick">showHide("<xsl:value-of select="name" />")</xsl:attribute> 
						<td class="feature"><xsl:value-of select="name"/></td>
						<td class="description"><xsl:value-of select="description"/></td>
						<td class="count"><xsl:value-of select="count"/></td>
						<!-- removed effort estimates -->
						<!-- <td class="effort"><xsl:apply-templates select="effort"/></td> -->
					</tr>
					<tr class="detail_container"><td colspan="4" class="detail_container">
						<table class="hide">
						<xsl:attribute name="id">
							<xsl:value-of select="name"/>
						</xsl:attribute>
			
						<xsl:for-each select="instances/instance">
						<tr class="detail">
							<td class="lineno">
										Line <xsl:value-of select="lineno"/>
							</td>
							<!--
							<td class="package">Package: <xsl:value-of select="package"/></td>
							-->
							<td class="object"><xsl:apply-templates select="object"/></td>
							<td class="context"><xsl:value-of select="context"/></td>
						</tr>
						</xsl:for-each>
						
						<xsl:if test="not(solution='')">
							<tr class="detail">
								<td valign="top" class="solution">Solution:</td>
								<td class="solution" colspan="2"><xsl:value-of select="solution"/></td>
							</tr>
						</xsl:if>
						</table>
					</td></tr>
				</xsl:for-each>
			</table>
		</xsl:if>


		<!-- removed for beta1 - note: had to comment out in several places due to internal comments -->
		<!--
		<a name="ddl_grouped_by_objects"/>
		<h3>DDL Grouped By Objects (Count: <xsl:value-of select="count(ddl/objects/object)"/>)</h3>
			<p class="displayMenu">
			<a href="javascript: displayDetailsInTable('detailedDdlTableByObjects')">Show Details</a> | 
			<a href="javascript: hideDetailsInTable('detailedDdlTableByObjects')">Hide Details</a>
		</p>
		<table id="detailedDdlTableByObjects">
			<thead>
				<th class="object">Object</th>
				<th class="count">#</th>
		-->
				<!-- removed effort estimates -->
				<!-- <th class="effort">Effort (person hours)</th> -->
		<!--
			</thead>
			<xsl:for-each select="ddl/objects/object">
				<xsl:sort select="object/@type"/>
				<xsl:sort select="object"/>
				<tr class="summary">
					<xsl:attribute name="onClick">showHide("<xsl:value-of select="object" />")</xsl:attribute> 
					<td class="object"><xsl:apply-templates select="object"/></td>
					<td class="count"><xsl:value-of select="count"/></td>
		-->
					<!-- removed effort estimates -->
					<!-- <td class="effort"><xsl:apply-templates select="effort"/></td> -->
		<!--
				</tr>
				<tr class="detail_container"><td colspan="5" class="detail_container">
				<table class="hide">
					<xsl:attribute name="id">
						<xsl:value-of select="object"/>
					</xsl:attribute>
								
					<xsl:for-each select="instances/instance">
						<tr class="detail">
		-->
							<!-- removed and replaced with item below for reading ddl from file instead of DB connection -->
							<!--
							<td class="location"><xsl:apply-templates select="location"/></td>
							<td class="feature"><xsl:value-of select="feature"/></td>
							-->
		<!--
							<xsl:call-template name="plsqlInstanceByOject">
								<xsl:with-param name="instance" select="."/>
							</xsl:call-template>
		-->
							<!-- removed effort estimates -->
							<!-- <td class="effort"><xsl:apply-templates select="effort"/></td> -->
		<!--
						</tr>
					</xsl:for-each>
				</table>
				</td></tr>
			</xsl:for-each>
		</table>
		-->
		
		<a name="unrecognized_grammars"/>
		<h3>Unrecognized Grammars (Count: <xsl:value-of select="count(unrecognized_grammars/instance)"/>)</h3>
		<xsl:choose>
			<xsl:when test="count(unrecognized_grammars/instance) > 0">
				<p>
					MEET DB2 fails to understand the following lines in the supplied Procedural SQL code.
				</p>
				
				<table id="unrecognizedGrammars">
					<thead>
						<th class="lineno">Line</th>
						<th class="error">Error</th>
						<th class="context">Context</th>
					</thead>
					
					<xsl:for-each select="unrecognized_grammars/instance">
						<tr class="detail">
							<td class="lineno">
									Line <xsl:value-of select="lineno"/>
							</td>
							<td class="message"><xsl:value-of select="message"/></td>
							<td class="context"><xsl:value-of select="context"/></td>
						</tr>
					</xsl:for-each>
				</table>
			</xsl:when>
			<xsl:otherwise>
				<!-- no unrecognized grammars -->
				<p>
					MEET DB2 fully parses the supplied Procedural SQL source code.
				</p>
			</xsl:otherwise>
		</xsl:choose>
		
		<h3>Version information</h3>
		<b>MEET Build date: </b> 
			<xsl:choose>
				<xsl:when test="exists(executeinfo/builddate)">
					<xsl:value-of select="executeinfo/builddate" />
				</xsl:when>
				<xsl:otherwise>
					2009-12-30
				</xsl:otherwise>
			</xsl:choose>
		<br/>
		<b>DB2 version: </b> 
			<xsl:choose>
				<xsl:when test="exists(executeinfo/version/query)">
					<xsl:value-of select="executeinfo/version/query" />
				</xsl:when>
				<xsl:otherwise>
					9.7 FP1
				</xsl:otherwise>
			</xsl:choose>
		<br/>
		<b>MEET version: </b> 
			<xsl:choose>
				<xsl:when test="exists(executeinfo/version/meet)">
					<xsl:value-of select="executeinfo/version/meet" />
				</xsl:when>
				<xsl:otherwise>
					0.51
				</xsl:otherwise>
			</xsl:choose>
		<br/>
	
	</div>
	
</xsl:template>


<xsl:template name="statisticsTable">
	<xsl:param name="title"/>
	<xsl:param name="items"/>
	<xsl:param name="text"/>
	
	<h3><xsl:value-of select="$title"/></h3>
	<p><xsl:value-of select="$text"/></p>
	
	<table class="statistics">
		<xsl:for-each select="$items/item[not(@display = 'no')]">
			<tr>
				<td><xsl:value-of select="description"/></td>
				<td><xsl:value-of select="result"/></td>
			</tr>
		</xsl:for-each>
	</table>
</xsl:template>


<xsl:template name="ddlInstance">
	<xsl:param name="instance"/>
	
</xsl:template>

<xsl:template name="plsqlInstanceByOject">
	<xsl:param name="instance"/>
	
	<tr class="detail">
		<td class="lineno">Line <xsl:value-of select="$instance/lineno"/></td>
		<td class="feature"><xsl:value-of select="$instance/feature"/></td>
		<td class="context"><xsl:value-of select="$instance/context"/></td>
		<!-- removed effort estimates -->
		<!-- <td class="effort"><xsl:apply-templates select="$instance/effort"/></td> -->
	</tr>
</xsl:template>


<xsl:template match="object">
	[<xsl:value-of select="@type"/>]
	<xsl:value-of select="text()"/>
</xsl:template>


<!-- removed effort estimates -->
<!-- 
<xsl:template match="effort">
	<xsl:value-of select="low"/> - <xsl:value-of select="high"/>
</xsl:template>
-->

<xsl:template match="location">
	<xsl:if test="view_name">
		View <xsl:value-of select="view_name"/>
	</xsl:if>
	<xsl:if test="table_name">
		Table <xsl:value-of select="table_name"/>
	</xsl:if>
	<xsl:if test="column_name">
		Column <xsl:value-of select="column_name"/>
	</xsl:if>
</xsl:template>


</xsl:stylesheet>

