<?xml version="1.0" encoding="ISO-8859-1"?>

<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:meet="java:com.ibm.extremeblue.meet.xutils.MeetXUtils">

<xsl:output method="html" version="4.0" encoding="ISO-8859-1" doctype-public="-//W3C//DTD XHTML 1.1//EN" indent="yes"/>

<xsl:key name="symbol-key" match="error" use="symbol"/>

<xsl:template match="errors">

  <html>
    <style>
      <xsl:value-of select="meet:loadParsingErrorsCSS()"/>
    </style>

    <script>
      <xsl:value-of select="meet:loadParsingErrorsJavascript()"/>
    </script>

    <head>
      <title>Parsing Error Report</title>
      <link rel="stylesheet" href="parsing-errors.css" type="text/css"></link>
      <script type="text/javascript" src="parsing-errors.js"/>
    </head>

    <body>
      <xsl:variable name="symbol-set" select="error[. = key('symbol-key', symbol)[1]]/symbol"/>

      <h1>Parsing Error Report</h1>
      <p>There were a total of <xsl:value-of select="count(error)"/> errors on <xsl:value-of select="count($symbol-set)"/> different symbols.  Click on a symbol below to show or hide the error details.</p>

      <table>
        <tr>
          <th>Symbol</th>
          <th>Errors</th>
        </tr>

      <xsl:for-each select="$symbol-set">
        <xsl:sort select="count(key('symbol-key', .))" data-type="number" order="descending"/>

        <xsl:variable name="id" select="generate-id(.)"/>
        <xsl:variable name="count" select="count(key('symbol-key', .))"/>

        <tr id="{$id}-show" class="show-errors" onClick="showDetails('{$id}')">
          <td><xsl:value-of select="."/></td>
          <td><xsl:value-of select="$count"/></td>
        </tr>

        <tr id="{$id}-hide" class="hide-errors" onClick="hideDetails('{$id}')">
          <td><xsl:value-of select="."/></td>
          <td><xsl:value-of select="$count"/></td>
        </tr>
        
        <tr id="{$id}" class="symbol-errors">
          <td colspan="2">
            <table id="{$id}">
              <tr>
                <th>File</th>
                <th>Line</th>
                <th>Context</th>
                <th>Message</th>
              </tr>

              <xsl:for-each select="key('symbol-key', .)">
                <tr>
                  <td><a href="file:///{filePath}" target="_blank"><xsl:value-of select="fileName"/></a></td>
                  <td><xsl:value-of select="lineNumber"/></td>
                  <td class="code"><xsl:value-of select="context"/></td>
                  <td><xsl:value-of select="message"/></td>
                </tr>
              </xsl:for-each>

            </table>
          </td>
        </tr>
        
      </xsl:for-each>
      </table>
    </body>

  </html>
</xsl:template>

</xsl:stylesheet>