<?xml version="1.0"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:meet="java:com.ibm.extremeblue.meet.xutils.MeetXUtils">

<!-- Generates a summary in CSV format of incompatible statements, objects, features and their counts. -->
<!-- First line contains: filename, db2 version, num statements, num incompatible statements,num objects, num incompatible objects -->
<!-- Each subsequent lines contain: filename, db2 version, feature, count -->

<!-- set for text output, rather than xml output -->
<xsl:output method="text"/>

<!-- set path delimeter - should be \ or / -->
<xsl:variable name="PATH_DELIMETER" select="'\'"/>

<xsl:template match="report">
	<!-- set up variables: -->

	<!-- get and store filepath -->
	<xsl:variable name="filepath" select="sources/source[type='File']/name" />
	
	<!-- extract and store filename -->
	<xsl:variable name="filename">
		<xsl:call-template name="getFilename">
			<xsl:with-param name="filepath" select="$filepath"/>
		</xsl:call-template>
	</xsl:variable>
	
	<!-- retrieve and store db2 version -->
	<xsl:variable name="db2version"  select="executeinfo/version/query"/>

	<!-- output: -->
	
	<!-- statement and object counts -->
	<!-- filename -->
	<xsl:copy-of select="$filename" />
	<xsl:text>,</xsl:text>
	<!-- db2 version -->
	<xsl:copy-of select="$db2version" />
	<xsl:text>,</xsl:text>
	<!-- number of statements -->
	<xsl:value-of select="summary/item[object_type = 'Statements']/number"/>
	<xsl:text>,</xsl:text>
	<!-- number of incompatible statements -->
	<xsl:value-of select="summary/item[object_type = 'Statements']/number_incompatible"/>
	<xsl:text>,</xsl:text>
	<!-- number of objects -->
	<xsl:value-of select="summary/item[object_type = 'Total Objects']/number"/>
	<xsl:text>,</xsl:text>
	<!-- number of incompatible objects -->
	<xsl:value-of select="summary/item[object_type = 'Total Objects']/number_incompatible"/>
	<!-- new line -->
	<xsl:text>&#10;</xsl:text>

	<!-- features -->
	<!-- for all features -->
	<xsl:for-each select="plsql/features/feature">
		<!-- filename -->
		<xsl:copy-of select="$filename" />
		<xsl:text>,</xsl:text>
		<!-- db2 version -->
		<xsl:copy-of select="$db2version" />
		<xsl:text>,</xsl:text>
		<!-- feature name -->
		<xsl:value-of select="name"/>
		<xsl:text>,</xsl:text>
		<!-- feature count -->
		<xsl:value-of select="count"/>
		<!-- new line -->
		<xsl:text>&#10;</xsl:text>
	</xsl:for-each>
</xsl:template>



<!-- helper templates/functions: -->

<!-- getFilename - extract the filename from a full file path -->
<xsl:template name="getFilename">
	<xsl:param name="filepath"/>
	
	<xsl:if test="contains($filepath, $PATH_DELIMETER)">
	
		<xsl:variable name="fileLength" select="string-length($filepath)"/>
		
		<xsl:variable name="delemiterPosition">
			<xsl:call-template name="getDelemiterPosition">
				<xsl:with-param name="filepath" select="$filepath"/>
				<xsl:with-param name="startSearchPosition"
						select="$fileLength"/>
			</xsl:call-template>
		</xsl:variable>
		
		<xsl:value-of select="substring($filepath,
						$delemiterPosition + 1,
						$fileLength - $delemiterPosition + 1)"/>

	</xsl:if>
</xsl:template>

<!-- getDelemiterPosition - helper function for getFilename -->
<xsl:template name="getDelemiterPosition">
	<xsl:param name="filepath"/>
	<xsl:param name="startSearchPosition"/>
	
	<xsl:if test="$startSearchPosition > 0">
		<xsl:choose>
			<xsl:when test="substring($filepath, $startSearchPosition, 1) = $PATH_DELIMETER">
				<xsl:value-of select="$startSearchPosition"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="getDelemiterPosition">
					<xsl:with-param name="filepath" select="$filepath"/>
					<xsl:with-param name="startSearchPosition"
							select="$startSearchPosition - 1"/>
				</xsl:call-template>		
			</xsl:otherwise>
		</xsl:choose>
	</xsl:if>
</xsl:template>

</xsl:stylesheet>
