<?xml version="1.0" encoding="ISO-8859-1"?>

<xsl:stylesheet version="2.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:meet="java:com.ibm.extremeblue.meet.xutils.MeetXUtils">

<xsl:template match="report" mode="technical_report">

	<a name="technical_summary"/>
	<h2>Technical Summary</h2>

	<p>
		<b>Report generated on: </b>
		<xsl:variable name="reportdatetime" select="executeinfo/datetime" />
		<xsl:value-of select="format-dateTime($reportdatetime, '[Y0001]/[M01]/[D01]')" />
		<xsl:text> at </xsl:text>
		<xsl:value-of select="format-dateTime($reportdatetime, '[h]:[m01] [Pn]')" />
	</p>
	
	<h3>Source</h3>
	<ul>
		<xsl:for-each select="sources/source">
			<li><b><xsl:value-of select="type"/>:</b><xsl:text>&#160;</xsl:text><xsl:value-of select="name"/></li>
		</xsl:for-each>
	</ul>
	
	<h3>Summary</h3>
	<table id="summary">
		<thead>
			<tr>
				<th>Object Type</th>
				<th>Total Number</th>
				<th>Number That Require Attention</th>
				<th>Percent That Require Attention</th>
			</tr>
		</thead>
		<tbody>
			<!-- first pull out the detail elements, so that they can be sorted -->
			<xsl:for-each select="summary/item">
				<xsl:sort select="percent_incompatible" data-type="number" order="ascending"/>
				<xsl:sort select="number" data-type="number" order="ascending"/>
				<xsl:if test="object_type != 'Total Objects' and object_type != 'Statements' and object_type != 'Total Native Components' 
									and object_type != 'function call' and object_type != 'stored procedure' and object_type != 'system variable'">
					<tr class="detail">
						<td><xsl:value-of select="object_type"/></td>
						<td><xsl:value-of select="number"/></td>
						<td><xsl:value-of select="number_incompatible"/></td>
						<td><xsl:value-of select="percent_incompatible"/> %</td>
					</tr>
				</xsl:if>
			</xsl:for-each>
			<!-- then, pull out the summary elements, so they'll be at the bottom -->
			<xsl:for-each select="summary/item">
				<xsl:if test="object_type = 'Total Objects' or object_type = 'Statements'">
					<tr class="summary">
						<td><xsl:value-of select="object_type"/></td>
						<td><xsl:value-of select="number"/></td>
						<td><xsl:value-of select="number_incompatible"/></td>
						<td><xsl:value-of select="percent_incompatible"/> %</td>
					</tr>
				</xsl:if>
			</xsl:for-each>
		</tbody>
	</table>

	<p>
		<a class="show" id="showTechnicalReport" href="javascript:showHide('technical_details');showObject('hideTechnicalReport');hideObject('showTechnicalReport');">Click here to show the detailed technical report</a>
		<a class="hide" id="hideTechnicalReport" href="javascript:showHide('technical_details');showObject('showTechnicalReport');hideObject('hideTechnicalReport');">Click here to hide the detailed technical report</a>
	</p>
	
	<div id="technical_details" class="hide">
		
		<!-- Generate PL/SQL Statistics -->
		<a name="plsql_statistics"/>
		<xsl:call-template name="statisticsTable">
			<xsl:with-param name="title">Statistics</xsl:with-param>
			<xsl:with-param name="items" select="plsql/statistics"/>
		</xsl:call-template>

		<a name="plsql_grouped_by_features"/>
		<h3>Features (Count: <xsl:value-of select="count(plsql/features/feature[@informational='false'])"/>)</h3>
		<p class="displayMenu">
			<a href="javascript: displayDetailsInTable('detailedPlsqlTableByFeatures')">Show Details</a> | 
			<a href="javascript: hideDetailsInTable('detailedPlsqlTableByFeatures')">Hide Details</a>
			(Or click on an individual item to show its details)
		</p>
		<table id="detailedPlsqlTableByFeatures">
			<thead>
				<th class="feature">Feature</th>
				<th class="description">Description</th>
				<th class="count">#</th>
			</thead>
			<xsl:for-each select="plsql/features/feature[@informational='false']">
				<tr class="summary">
					<xsl:attribute name="onClick">showHide("<xsl:value-of select="name" />")</xsl:attribute> 
					<td class="feature"><xsl:value-of select="name"/></td>
					<td class="description"><xsl:value-of select="description"/></td>
					<td class="count"><xsl:value-of select="count"/></td>
				</tr>
				<tr class="detail_container"><td colspan="4" class="detail_container">
					<table class="hide">
						<xsl:attribute name="id">
							<xsl:value-of select="name"/>
						</xsl:attribute>
			
						<tr class="detail">
							<td class="effort_sybase">Effort:</td>
							<td class="effort_sybase" colspan="2"><xsl:value-of select="effort"/></td>
						</tr>

						<xsl:if test="not(solution='')">
							<tr class="detail">
								<td valign="top" class="solution_sybase">Solution:</td>
								<td class="solution_sybase" colspan="2"><xsl:value-of select="solution"/></td>
							</tr>
						</xsl:if>
	
						<xsl:for-each select="instances/instance">
							<tr class="detail">
								<td class="lineno">
									Line <xsl:value-of select="lineno"/>
								</td>
								<td class="object"><xsl:apply-templates select="object"/></td>
								<td class="context"><xsl:value-of select="context"/></td>
							</tr>
						</xsl:for-each>
					</table>
				</td></tr>
			</xsl:for-each>
		</table>

		<xsl:if test="count(plsql/features/feature[@informational='true']) > 0">
			<a name="plsql_informational_features"/>
			<h3>Features - Informational (Count: <xsl:value-of select="count(plsql/features/feature[@informational='true'])"/>)</h3>
			<p class="displayMenu">
				<a href="javascript: displayDetailsInTable('detailedPlsqlTableInformationalFeatures')">Show Details</a> | 
				<a href="javascript: hideDetailsInTable('detailedPlsqlTableInformationalFeatures')">Hide Details</a>
				(Or click on an individual item to show its details)
			</p>
			<table id="detailedPlsqlTableInformationalFeatures">
				<thead>
					<th class="feature">Feature</th>
					<th class="description">Description</th>
					<th class="count">#</th>
				</thead>
				<xsl:for-each select="plsql/features/feature[@informational='true']">
					<tr class="summary">
						<xsl:attribute name="onClick">showHide("<xsl:value-of select="name" />")</xsl:attribute> 
						<td class="feature"><xsl:value-of select="name"/></td>
						<td class="description"><xsl:value-of select="description"/></td>
						<td class="count"><xsl:value-of select="count"/></td>
					</tr>
					<tr class="detail_container"><td colspan="4" class="detail_container">
						<table class="hide">
							<xsl:attribute name="id">
								<xsl:value-of select="name"/>
							</xsl:attribute>
				
							<tr class="detail">
								<td class="effort_sybase">Effort:</td>
								<td class="effort_sybase" colspan="2"><xsl:value-of select="effort"/></td>
							</tr>
	
							<xsl:if test="not(solution='')">
								<tr class="detail">
									<td valign="top" class="solution_sybase">Solution:</td>
									<td class="solution_sybase" colspan="2"><xsl:value-of select="solution"/></td>
								</tr>
							</xsl:if>
	
							<xsl:for-each select="instances/instance">
								<tr class="detail">
									<td class="lineno">
										Line <xsl:value-of select="lineno"/>
									</td>
									<td class="object"><xsl:apply-templates select="object"/></td>
									<td class="context"><xsl:value-of select="context"/></td>
								</tr>
							</xsl:for-each>
						</table>
					</td></tr>
				</xsl:for-each>
			</table>
		</xsl:if>
		
		<p>
			<b>Estimated Effort levels</b>
			<ul>
				<li>0 - There is no effort involved, the feature is purely informational</li>
				<li>1 - It is possible, but unlikely, that the feature will cause any application issues and/or they are very easy to remediate</li>
				<li>2 - Some aspect of the feature may not be fully supported or may need to be reviewed to see if there may be some implications of its use</li>
				<li>3 - Simple modification will be required for this feature</li>
				<li>4 - Moderate modification will be required for this feature</li>
				<li>5 - Substantial modification will be required for this feature</li>
			</ul>
		</p>
		
		<a name="unrecognized_grammars"/>
		<h3>Unrecognized Grammars (Count: <xsl:value-of select="count(unrecognized_grammars/instance)"/>)</h3>
		<xsl:choose>
			<xsl:when test="count(unrecognized_grammars/instance) > 0">
				<p>
					MEET DB2 fails to understand the following lines in the supplied SQL code.
				</p>
				
				<table id="unrecognizedGrammars">
					<thead>
						<th class="lineno">Line</th>
						<th class="error">Error</th>
						<th class="context">Context</th>
					</thead>
					
					<xsl:for-each select="unrecognized_grammars/instance">
						<tr class="detail">
							<td class="lineno">
									Line <xsl:value-of select="lineNumber"/>
							</td>
							<td class="message"><xsl:value-of select="message"/></td>
							<td class="context"><xsl:value-of select="context"/></td>
						</tr>
					</xsl:for-each>
				</table>
			</xsl:when>
			<xsl:otherwise>
				<!-- no unrecognized grammars -->
				<p>
					MEET DB2 fully parses the supplied SQL source code.
				</p>
			</xsl:otherwise>
		</xsl:choose>


		<h3>Version information</h3>
		<b>MEET Build date: </b> 
		<xsl:value-of select="executeinfo/builddate" />
		<br/>
		<b>DB2 version: </b> 
		<xsl:value-of select="executeinfo/version/query" />
		<br/>
		<b>MEET version: </b> 
		<xsl:value-of select="executeinfo/version/meet" />
		<br/>

	</div>
	
</xsl:template>


<xsl:template name="statisticsTable">
	<xsl:param name="title"/>
	<xsl:param name="items"/>
	
	<h3><xsl:value-of select="$title"/></h3>
	
	<table class="statistics">
		<xsl:for-each select="$items/item[not(@display = 'no')]">
			<tr>
				<td><xsl:value-of select="description"/></td>
				<td><xsl:value-of select="result"/></td>
			</tr>
		</xsl:for-each>
	</table>
</xsl:template>


<xsl:template name="ddlInstance">
	<xsl:param name="instance"/>
	
</xsl:template>

<xsl:template name="plsqlInstanceByOject">
	<xsl:param name="instance"/>
	
	<tr class="detail">
		<td class="lineno">Line <xsl:value-of select="$instance/lineno"/></td>
		<td class="feature"><xsl:value-of select="$instance/feature"/></td>
		<td class="context"><xsl:value-of select="$instance/context"/></td>
	</tr>
</xsl:template>


<xsl:template match="object">
	[<xsl:value-of select="@type"/>]
	<xsl:value-of select="text()"/>
</xsl:template>


<xsl:template match="location">
	<xsl:if test="view_name">
		View <xsl:value-of select="view_name"/>
	</xsl:if>
	<xsl:if test="table_name">
		Table <xsl:value-of select="table_name"/>
	</xsl:if>
	<xsl:if test="column_name">
		Column <xsl:value-of select="column_name"/>
	</xsl:if>
</xsl:template>


</xsl:stylesheet>

