#!/usr/bin/perl -w
#
# 
#  This is a SPECIAL PURPOSE script to group create/alter statements
#   and split them into separate files.
#  It was written for a particular input file
#    so do not complain if it does not work for other files.
#  (And even for that particular file this script
#     is provided as is, without guarantees)
# 
#
# @Author: Jannis Dorloechter (jannis.dorloechter@de.ibm.com)
#
# Copyright IBM 2010.
#
#

use strict;
use utf8;


sub usage()
{
	print "Usage: $0 <filename> [linesperfile]\n";
}

if( $#ARGV == -1 or $#ARGV > 1 )
{
	usage();
	exit 1;
}

my $filename = $ARGV[0];


open(INPUT, "<$filename") or die "error opening file $filename";


open(TABLEOUTPUT, ">$filename\_TABLES_00");
open(INDEXOUTPUT, ">$filename\_INDICES_00");
open(PCKGOUTPUT, ">$filename\_PACKAGES_00");

my $tabfilecnt = 0;
my $tablinecnt = 0;
my $indexfilecnt = 0;
my $indexlinecnt = 0;
my $pckglinecnt = 0;
my $pckgfilecnt = 0;

my $totallinecount = 0;
my $printlinecount = 0;

# this is a very approximate value, because it waits for the end of the current item after that line 
#  (for a package there will probably be A LOT more lines)
my $LINESPERFILE = 10000;
if( $#ARGV == 1 )
{
	if( $ARGV[1] =~ /[0-9]+/ && $ARGV[1] > 10 )
	{
		$LINESPERFILE = $ARGV[1];
	}
	else
	{
		print "second argument is not a valid number\n";
		usage();
		exit 1;
	}
}

# by default put into tables file
my $state = 1;

while( my $line = <INPUT> )
{
	$totallinecount++;

	if( $line =~ /^\s*(CREATE|ALTER)\s+TABLE/i )
	{
		$state = 1;
		$tablinecnt++;
		
	}
	elsif( $line =~ /^\s*(CREATE|ALTER)\s+UNIQUE{0,1}\s+INDEX/i )
	{
		$state = 2;
		$indexlinecnt++;
	}
	elsif( $line =~ /^\s*CREATE\s+(OR REPLACE){0,1}\s+PACKAGE /i )
	{
		$state = 3;
		$pckglinecnt++;
	}
	
	# line that just contains a forward slash
	#elsif( $line =~ /^\s*\/[^\*]+\s*/)
	elsif( $line =~ /^\s*\/\s*$/)
	{
		if( $state == 1 )
		{
			$tablinecnt++;
			print TABLEOUTPUT $line; $printlinecount++;
			if($tablinecnt>=$LINESPERFILE)
			{
				print "$tablinecnt lines written to $filename\_TABLES\_$tabfilecnt\n";
				$tablinecnt = 0;
				close(TABLEOUTPUT);
				$tabfilecnt++;
				my $newfilename = "$filename\_TABLES\_";
				if( $tabfilecnt < 10 )
				{ $newfilename .= "0$tabfilecnt";}
				else
				{$newfilename .= $tabfilecnt;}
				open(TABLEOUTPUT, ">$newfilename") or die("error opening file: $newfilename");
			}
		}
		elsif( $state == 2 )
		{
			$indexlinecnt++;
			print INDEXOUTPUT $line;$printlinecount++;
			if( $indexlinecnt>=$LINESPERFILE)
			{
				print "$indexlinecnt lines written to $filename\_INDICES\_$indexfilecnt\n";
				$indexlinecnt=0;
				close(INDEXOUTPUT);
				$indexfilecnt++;
				my $newfilename = "$filename\_INDICES\_";
				if( $indexfilecnt < 10 )
				{ $newfilename .= "0$indexfilecnt";}
				else
				{$newfilename .= $indexfilecnt;}
				open(INDEXOUTPUT, ">$newfilename") or die("error opening file: $newfilename");
			}
		}
		elsif( $state == 3 )
		{
			$pckglinecnt++;
			
			print PCKGOUTPUT $line;$printlinecount++;
			if( $pckglinecnt>=$LINESPERFILE)
			{
				print "$pckglinecnt lines written to $filename\_PACKAGES\_$pckgfilecnt\n";
				$pckglinecnt=0;
				close(PCKGOUTPUT);
				$pckgfilecnt++;
				my $newfilename = "$filename\_PACKAGES\_";
				if( $pckgfilecnt < 10 )
				{ $newfilename .= "0$pckgfilecnt";}
				else
				{$newfilename .= $pckgfilecnt;}
				open(PCKGOUTPUT, ">$newfilename") or die("error opening file: $newfilename");
			}
		}
		
		# we already printed this line, go to next line
		next;

	}
	
	# print everything

	if( $state == 1 )
	{
		print TABLEOUTPUT $line;$printlinecount++;
		$tablinecnt++;
	}
	elsif( $state == 2 )
	{
		print INDEXOUTPUT $line;$printlinecount++;
		$indexlinecnt++;
	}
	elsif( $state == 3)
	{
		print PCKGOUTPUT $line;$printlinecount++;
		$pckglinecnt++;
	}

}

print "verifying that all lines were included ...";
if( $totallinecount != $printlinecount )
{
	print "FAILED!\n";
	print "Totallinecount != Printedlinecount: $totallinecount != $printlinecount\n";
}
else
{
	print "OK ($totallinecount lines)\n";
}

close(INPUT);
close(PCKGOUTPUT);
close(INDEXOUTPUT);
close(TABLEOUTPUT);
