module namespace effort = "http://extremeblue.ibm.com/meet/effort";

(::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 : Returns the sum of two effort tags.
 :
 : For example,
 :   effort:add(
 :     <effort><low>2</low><high>3</high></effort>,
 :     <effort><low>3</low><high>5</high></effort>)
 : would return
 :   <effort><low>5</low><high>8</high></effort>
 :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::)
declare function effort:add ($effort1, $effort2)
{
	<effort>
		<low>{ $effort1/low/text() + $effort2/low/text() }</low>
		<high>{ $effort1/high/text() + $effort2/high/text() }</high>
	</effort>
};

(::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 : Returns the sum of a sequence of effort tags.
 : 
 : For example, effort:sum would return 
 :   <effort><low>12</low><high>23</high></effort>
 : if the following sequence is passed as the argument
 :   <effort><low>1</low><high>1</high></effort>
 :   <effort><low>2</low><high>6</high></effort>
 :   <effort><low>6</low><high>5</high></effort>
 :   <effort><low>3</low><high>11</high></effort>
 :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::)
declare function effort:sum ($efforts)
{
	<effort>
		<low>{ sum($efforts//low/text()) }</low>
		<high>{ sum($efforts//high/text()) }</high>
	</effort>
};

(::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 : Multiplies the effort tag by multiplier.
 : 
 : For example,
 :   effort:add(
 :     <effort><low>2</low><high>3</high></effort>,
 :     3)
 : would return
 :   <effort><low>6</low><high>9</high></effort>
 :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::)
declare function effort:multiply ($effort, $multiplier as xs:decimal)
{
	<effort>
		<low>{ $effort/low/text() * $multiplier }</low>
		<high>{ $effort/high/text() * $multiplier }</high>
	</effort>
};

(::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 : Returns an effort tag with the supplied $low/$high effort values.
 : 
 : For example, effort:add(3, 4) would return
 :   <effort><low>3</low><high>4</high></effort>
 :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::)
declare function effort:init ($low as xs:decimal, $high as xs:decimal)
{
	<effort>
		<low>{ $low }</low>
		<high>{ $high }</high>
	</effort>
};

(::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 : Returns an effort tag with 0 as the low and high effort values.
 :   <effort><low>0</low><high>0</high></effort>
 :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::)
declare function effort:zero ()
{
	effort:init(0, 0)
};

