module namespace meet = "http://extremeblue.ibm.com/meet/meet";
declare namespace math = "java:java.lang.Math";

(::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 : Returns true if there is a node in $seq that has the same text 
 : value as $node, and false otherwise. This is mainly used as a
 : helper function for meet:distinct-nodes(). For the purpose of MEET,
 : we are testing the text value of nodes rather than XML node
 : identities.
 :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::)
declare function meet:is-node-in-sequence ($node as node()?, $seq as node()* )
as xs:boolean
{
	if ($node/text())
	then some $nodeInSeq in $seq satisfies $nodeInSeq/text() = $node/text()
	else some $nodeInSeq in $seq satisfies not($nodeInSeq/text())
};

(::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 : Removes duplicates in $nodes and returns them.
 :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::)
declare function meet:distinct-nodes ( $nodes as node()* ) as node()*
{
	(: Algorithm:
	 : Iterates through all nodes and only returns a node if it
	 : doesn't have a duplicate in the rest of the untouched nodes
	 :)
	for $seq in (1 to count($nodes))
	return $nodes[$seq][not(meet:is-node-in-sequence(., $nodes[position() < $seq]))]
};

(::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 : Returns the rounded percentage value of $numerator/$denominator.
 :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::)
declare function meet:percent ($numerator, $denominator)
{
	if ($denominator = 0) then 0
	else round(($numerator * 100) div $denominator)
};

(::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 : Returns the percentage value of $numerator/$denominator rounded to the specified number of decimal places.
 :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::)
declare function meet:percent-decimalplaces ($numerator, $denominator, $decimalplaces)
{
	if ($denominator = 0) then 0
	else round((($numerator * 100) div $denominator) * math:pow(10, $decimalplaces)) div math:pow(10, $decimalplaces) 
};
