module namespace ast = "http://extremeblue.ibm.com/meet/ast";


(::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 : Returns the tag for an identified unsupported instance. 
 :   <instance>
 :     <lineno> [ line number of the instance ] </lineno>
 :     <object type="[type (function, procedure, trigger) of the object this instance belongs to]">
 :       [name of the object this instance belongs to]
 :     </object>
 :     <statementpath>
 :       [path in ast.xml of the statement node that contains this instance]
 :     </statementpath>
 :   </instance>
 : Statement path may not appear, for example for instances of DDL incompatibilities.
 : If @objtype="" then it is set to "anonymous".
 :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::)
declare function ast:reportNode($node, $statementpath) as element() {
	<instance>
		<lineno>{ $node/data(@lineno) }</lineno>
		<object type="{
		if ($node/data(@objtype) != "")
		then $node/data(@objtype)
		else "anonymous"
		}">{ $node/data(@object) }</object>
		<statementpath>{ $statementpath }</statementpath>
	</instance>
};
(: Same function as above but without $statementpath :)
declare function ast:reportNode($node) as element() {
	<instance>
		<lineno>{ $node/data(@lineno) }</lineno>
		<object type="{
		if ($node/data(@objtype) != "")
		then $node/data(@objtype)
		else "anonymous"
		}">{ $node/data(@object) }</object>
	</instance>
};

(::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 : Returns the number of lines of code for an AST node $node.
 :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::)
declare function ast:getNodeLOC($node) {
    (: The implementation of this function is not 100% accurate. We calculate
     : the LOC from the line number of $node to the largest line number found
     : in all child nodes of $node. 
     :)
	let $children := $node/descendant::*
	return(max(( max($children/data(@lineno)), max($children/data(@lastlineno)) )) + 1 - $node/data(@lineno))
};

(::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 : Returns the number of parameters in a program (function or
 : procedure) definition. This is a helper function for PL/SQL
 : incompatible feature XQueries or statistics XQueries. 
 :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::)
declare function ast:getNumberParameters($node) {
	let $parameters := $node//node[@type="formalArg"]
	return count($parameters) 
};

(::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 : The number of plsql statements is anything that is not one of the below.
 : Note that this list of conditions is also used in the param passed to function ast:reportNode above, so
 : any changes to the conditions here should also be made to any calls to reportNode($node, $statementpath) as well.
 :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::)
declare function ast:getNumberOfStatements($node) {
	count($node//node[@type != "token" and @type !="list" and @type != "ddl" and 
					  @type != "block" and @type != "colDecl" and @type != "formalArg"])
};

(::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 : The number of ddl statements is any statement with type = "ddl" 
 : vardecl in a table definition
 :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::)
declare function ast:getNumberOfDDLStatements($node) {
	count($node//node[@type = "ddl"])
};


