
import module namespace effort = "http://extremeblue.ibm.com/meet/effort" at "xquery/effort.xq";
import module namespace meet   = "http://extremeblue.ibm.com/meet/meet"   at "xquery/meet.xq";
import module namespace ast    = "http://extremeblue.ibm.com/meet/ast"    at "xquery/oracle/ast.xq";


declare namespace meetjava="java:com.ibm.extremeblue.meet.xutils.MeetXUtils";
declare namespace console="java:com.ibm.extremeblue.meet.xutils.Console";

declare variable $knowledgebase external;

declare variable $preliminary_plsql_report external;
(: declare variable $preliminary_ddl_report external; :)

declare function local:getIncompatibleFeatures($preliminary_plsql_report)
{
	<features>
		{
		for $feature in $preliminary_plsql_report//features/feature
		where local:getFeatureInformational($feature/@id)="false"
		return $feature
		}
	</features>
};

declare function local:getStatisticsItemResult($preliminary_report, $id)
{
	if ($preliminary_report//statistics/item[@id = upper-case($id)]/result/text())
	then $preliminary_report//statistics/item[@id = upper-case($id)]/result/text()
	else 0
};

declare function local:getStatisticsItemDesc($preliminary_report, $id)
{
	if ($preliminary_report//statistics/item[@id = upper-case($id)]/description/text())
	then $preliminary_report//statistics/item[@id = upper-case($id)]/description/text()
	else 0
};

declare function local:getStatisticsItemSourceResult($preliminary_report, $id)
{
	if ($preliminary_report//statistics/item[@id = upper-case($id)]/result/text())
	then $preliminary_report//statistics/item[@id = upper-case($id)]/result/text()
	else "none"
};

declare function local:getFeatureName ($feature_id as xs:string)
{
	if ($knowledgebase//feature[@id=$feature_id]/name)
	then $knowledgebase//feature[@id=$feature_id]/name/text()
	else concat("undefined name for ", $feature_id)
};

declare function local:getFeatureEffort ($feature_id as xs:string)
{	
	if ($knowledgebase//feature[@id=$feature_id]/effort)
	then
		effort:init($knowledgebase//feature[@id=$feature_id]/effort/low/text(),
					$knowledgebase//feature[@id=$feature_id]/effort/high/text())
	else effort:zero()
};

declare function local:getFeatureDescription ($feature_id as xs:string)
as xs:string
{
	if ($knowledgebase//feature[@id=$feature_id]/description)
	then $knowledgebase//feature[@id=$feature_id]/description/text()
	else concat("undefined description for ", $feature_id)
};
 
declare function local:getFeatureSolution ($feature_id as xs:string)
as xs:string
{
	if ($knowledgebase//feature[@id=$feature_id]/solution)
	then $knowledgebase//feature[@id=$feature_id]/solution/text()
	else ""
};

declare function local:getFeatureInformational ($feature_id as xs:string)
as xs:string
{
	if ($knowledgebase//feature[@id=$feature_id]/@informational)
	then lower-case($knowledgebase//feature[@id=$feature_id]/@informational)
	else "false"
};

declare function local:generateInstanceTagsForPlsql ($instance)
{
	$instance/lineno, $instance/context, $instance/package
};

declare function local:generateInstanceTagsForDdl ($instance)
{
	if ($instance/location) then $instance/location
	else <location>{ $instance/table_name, $instance/column_name, $instance/index_name, $instance/view_name, $instance/type_name }</location>
};

declare function local:getDdlObjectInformation ($instance)
{
	if ($instance/table_name)
	then <object type="table">{ $instance/table_name/text() }</object>
	else if ($instance/index_name)
	then <object type="index">{ $instance/index_name/text() }</object>
	else if ($instance/view_name)
	then <object type="view">{ $instance/view_name/text() }</object>
	else if ($instance/type_name)
	then <object type="type">{ $instance/type_name/text() }</object>
	else <object type="null">unknown</object>
};

declare function local:addObjectInformationToDdlReport($preliminary_report)
{
	<report>
		{ $preliminary_report//statistics }
		<features>
		{
			for $feature in $preliminary_report/report/features/feature
			where $feature/instance
			return
				<feature id="{ $feature/data(@id) }">
				{
					for $instance in $feature/instance
					return
						<instance>
							{ local:generateInstanceTagsForDdl($instance) }
							{ local:getDdlObjectInformation($instance) }
						</instance>
				}
				</feature>
		}
		</features>
	</report>
};

declare function local:generateReportByFeatures ($preliminary_report, $type)
{
	<features>
	{
		(: detailed technical report grouped by features :)
		for $feature in $preliminary_report//features/feature[@type=$type]
		let $feature_id := $feature/data(@id)
		let $feature_name := local:getFeatureName($feature_id)
		let $count := count($feature/instance)
		let $effort := local:getFeatureEffort($feature_id)
		(: only report the feature if it has more than 0 instances  :)
		where $feature//instance
		return 
				<feature informational="{ local:getFeatureInformational($feature_id) }" type="{ $type }">
				<name>{ $feature_name }</name>
				<count>{ $count }</count>
				{ effort:multiply($effort, $count) }
				<description>{ local:getFeatureDescription($feature_id) }</description>
				<solution>{ local:getFeatureSolution($feature_id) }</solution>
				<instances>
				{
					for $instance in $feature/instance
					return
						<instance>
							{ $instance/object }
							{ local:generateInstanceTagsForPlsql($instance) }
							{ local:generateInstanceTagsForDdl($instance) }
						</instance>
				}
				</instances>
			</feature>
	}
	</features>
};

declare function local:generateReportByObjects ($preliminary_report, $type)
{
	<objects>
	{
		(: detailed technical report grouped by objects :)
		for $object in meet:distinct-nodes($preliminary_report/features/feature[@type=$type]//object)
		let $instances := $preliminary_report//instance[object = $object]
		let $count := count($instances)
		let $effort := effort:sum(
							for $instance in $instances 
							return local:getFeatureEffort($instance/../data(@id))
							)
		return 
			<object>
				{ $object }
				<count>{ $count }</count>
				{ $effort }
				<instances>
				{
					for $instance in $instances
					let $feature_id := data($instance/../@id)
					let $feature_name := local:getFeatureName($feature_id)
					let $feature_effort := local:getFeatureEffort($feature_id)
					return
					<instance>
						<feature>{ $feature_name }</feature>
						{ $feature_effort }
						{ local:generateInstanceTagsForPlsql($instance) }
						{ local:generateInstanceTagsForDdl($instance) }
					</instance>
				}
				</instances>
			</object>
	}
	</objects>
};

declare function local:generateSummaryItemForObjectType($preliminary_report, $incompatible_features, $object_type)
{
	let $number := local:getStatisticsItemResult($preliminary_report, concat('NUM_', upper-case($object_type)))
	let $description := local:getStatisticsItemDesc($preliminary_report, concat('NUM_', upper-case($object_type)))
	let $number_incompatible := count(meet:distinct-nodes($incompatible_features//object[@type = $object_type]))
	let $percent_incompatible := meet:percent($number_incompatible, $number)
	let $effort := effort:sum(
						for $instance in $preliminary_report//instance where $instance/object[@type = $object_type]
						return local:getFeatureEffort($instance/../data(@id))
						)
	return
		<item>
			<object_type>{ $description }</object_type>
			<number>{ $number }</number>
			<number_incompatible>{ $number_incompatible }</number_incompatible>
			<percent_incompatible>{ $percent_incompatible }</percent_incompatible>
			{ $effort }
		</item>
};

(: $preliminary_ddl_report := local:addObjectInformationToDdlReport($preliminary_ddl_report); :)

declare variable $plsql_report_by_features := local:generateReportByFeatures($preliminary_plsql_report, "plsql");
declare variable $plsql_report_by_objects  := local:generateReportByObjects($preliminary_plsql_report, "plsql");

(: declare variable $ddl_report_by_features := local:generateReportByFeatures($preliminary_ddl_report); :)
(: declare variable $ddl_report_by_objects  := local:generateReportByObjects($preliminary_ddl_report); :)
declare variable $ddl_report_by_features := local:generateReportByFeatures($preliminary_plsql_report, "ddl");
declare variable $ddl_report_by_objects  := local:generateReportByObjects($preliminary_plsql_report, "plsql");

declare variable $incompatible_features := local:getIncompatibleFeatures($preliminary_plsql_report);

declare variable $summary := (
	local:generateSummaryItemForObjectType($preliminary_plsql_report, $incompatible_features, 'package'),
	local:generateSummaryItemForObjectType($preliminary_plsql_report, $incompatible_features, 'function'),
	local:generateSummaryItemForObjectType($preliminary_plsql_report, $incompatible_features, 'procedure'),
	local:generateSummaryItemForObjectType($preliminary_plsql_report, $incompatible_features, 'trigger'),
	local:generateSummaryItemForObjectType($preliminary_plsql_report, $incompatible_features, 'anonymous')
);

(: The names used below must be the type= for the object :)
declare variable $ddlsummary := (
	local:generateSummaryItemForObjectType($preliminary_plsql_report, $incompatible_features, 'crtTab'),
	local:generateSummaryItemForObjectType($preliminary_plsql_report, $incompatible_features, 'crtIdx'),
	local:generateSummaryItemForObjectType($preliminary_plsql_report, $incompatible_features, 'crtTyp'),
	local:generateSummaryItemForObjectType($preliminary_plsql_report, $incompatible_features, 'crtVw'),
	local:generateSummaryItemForObjectType($preliminary_plsql_report, $incompatible_features, 'crtSeq'),
	local:generateSummaryItemForObjectType($preliminary_plsql_report, $incompatible_features, 'otherddl')
);

<report>

	<sources>
		<source>
			<type>File</type>
			<name>{ local:getStatisticsItemSourceResult($preliminary_plsql_report, 'FILE_PATH') }</name>
		</source>
	</sources>
	
	<executeinfo>
		<datetime>
			{ current-dateTime() }
		</datetime>
		<internalexternal>
			{ meetjava:getIsInternalExternal() }
		</internalexternal>
		<version>
			<query>
				{ meetjava:getConfig('QUERY_VERSION_NAME') }
			</query>
			<kb>
				{ meetjava:getConfig('ORACLE_KB_VERSION') }
			</kb>
			<xcompiler>
				{ meetjava:getConfig('XCOMPILER_VERSION') }
			</xcompiler>
			<meet>
				{ meetjava:getConfig('MEET_VERSION') }
			</meet>
		</version>
		<builddate>
			{ meetjava:getConfig('MEET_BUILD_DATE') }
		</builddate>
		<mode>Oracle</mode>
	</executeinfo>

<summary>
{
	$summary,
	<item>
		<object_type>Total Objects</object_type>
		<number>{ sum($summary/number) }</number>
		<number_incompatible>{ sum($summary/number_incompatible) }</number_incompatible>
		<percent_incompatible>{ meet:percent-decimalplaces(sum($summary/number_incompatible), sum($summary/number), 1) }</percent_incompatible>
		{ effort:sum($summary/effort) }
	</item>
	,
	<item>
		<object_type>Statements</object_type>
		<number>{ local:getStatisticsItemResult($preliminary_plsql_report, 'NUM_STATEMENT') }</number>
		<number_incompatible>{ count(distinct-values($incompatible_features//feature[@type!="ddl"]/instance/statementpath)) }</number_incompatible>
		<percent_incompatible>{
			meet:percent-decimalplaces(count(distinct-values($incompatible_features//feature[@type!="ddl"]/instance/statementpath)),
						local:getStatisticsItemResult($preliminary_plsql_report, 'NUM_STATEMENT'), 1)
		}</percent_incompatible>
		{ effort:sum($summary/effort) }
	</item>
}
</summary>

<ddlsummary>
{
	$ddlsummary,
	<item>
		<object_type>Total Objects</object_type>
		<number>{ sum($ddlsummary/number) }</number>
		<number_incompatible>{ sum($ddlsummary/number_incompatible) }</number_incompatible>
		<percent_incompatible>{ meet:percent-decimalplaces(sum($ddlsummary/number_incompatible), sum($ddlsummary/number), 1) }</percent_incompatible>
		{ effort:sum($ddlsummary/effort) }
	</item>
	,
	<item>
		<object_type>Statements</object_type>
		<number>{ sum($ddlsummary/number) }</number>
		<number_incompatible>{ sum($ddlsummary/number_incompatible) }</number_incompatible>
		<percent_incompatible>{
			meet:percent-decimalplaces(sum($ddlsummary/number_incompatible), sum($ddlsummary/number), 1)
		}</percent_incompatible>
		{ effort:sum($ddlsummary/effort) }
	</item>
}
</ddlsummary>

<plsql>
	<statistics>
		{$preliminary_plsql_report//statistics/item[@type="plsql"]}
	</statistics>
	{ $plsql_report_by_features }
	{ $plsql_report_by_objects }
</plsql>

<ddl>
	<!-- {$preliminary_ddl_report//statistics} -->
	<statistics>
		{$preliminary_plsql_report//statistics/item[@type="ddl"]}
	</statistics>
	{ $ddl_report_by_features }
	{ $ddl_report_by_objects }
</ddl>

{ $preliminary_plsql_report//unrecognized_grammars }

</report>
