module namespace ast = "http://extremeblue.ibm.com/meet/ast";


(::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 : Returns the tag for an identified unsupported instance. 
 :   <instance>
 :     <lineno> [ line number of the instance ] </lineno>
 :     <object type="[type (function, procedure, trigger) of the object this instance belongs to]">
 :       [name of the object this instance belongs to]
 :     </object>
 :   </instance>
 :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::)
declare function ast:reportNode($node) as element() {
	<instance>
		<lineno>{ $node/data(@lineno)  }</lineno>
		<statement>{ max($node/ancestor-or-self::*[@stmtID]/data(@stmtID)) }</statement>
		<object type="{ $node/data(@objtype) }">{ $node/data(@object) }</object>
	</instance>
};

(::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 : Returns the number of lines of code for an AST node $node.
 :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::)
declare function ast:getNodeLOC($node) {
    (: The implementation of this function is not 100% accurate. We calculate
     : the LOC from the line number of $node to the largest line number found
     : in all child nodes of $node. 
     :) 
	let $children := $node/descendant::*
	return ( max($children/data(@lineno))+1 - $node/data(@lineno) )
};

(::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 : Returns the number of parameters in a program (function or
 : procedure) definition. This is a helper function for PL/SQL
 : incompatible feature XQueries or statistics XQueries. 
 :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::)
declare function ast:getNumberParameters($node) {
	let $parameters := $node//node[@type="procArg"]
	return count($parameters) 
};


declare function ast:getNumberOfStatements($node) {
	count($node//node[@stmtID])
};


