
import module namespace meet   = "http://extremeblue.ibm.com/meet/meet"   at "xquery/meet.xq";

declare namespace meetjava="java:com.ibm.extremeblue.meet.xutils.MeetXUtils";
declare namespace console="java:com.ibm.extremeblue.meet.xutils.Console";


declare variable $knowledgebase external;
declare variable $preliminary_plsql_report external;


declare function local:getStatisticsItemResult($preliminary_report, $id)
{
	if ($preliminary_report//statistics/item[@id = upper-case($id)]/result/text())
	then $preliminary_report//statistics/item[@id = upper-case($id)]/result/text()
	else 0
};

declare function local:getFeatureName ($feature_id as xs:string)
{
	if ($knowledgebase//feature[@id=$feature_id]/name)
	then $knowledgebase//feature[@id=$feature_id]/name/text()
	else concat("undefined name for ", $feature_id)
};

declare function local:getFeatureDescription ($feature_id as xs:string)
as xs:string
{
	if ($knowledgebase//feature[@id=$feature_id]/description)
	then $knowledgebase//feature[@id=$feature_id]/description/text()
	else concat("undefined description for ", $feature_id)
};
 
declare function local:getFeatureSolution ($feature_id as xs:string)
as xs:string
{
	if ($knowledgebase//feature[@id=$feature_id]/solution and $knowledgebase//feature[@id=$feature_id]/solution/text() != '')
	then $knowledgebase//feature[@id=$feature_id]/solution/text()
	else ""
};

declare function local:getFeatureInformational ($feature_id as xs:string)
as xs:string
{
	if ($knowledgebase//feature[@id=$feature_id]/@informational)
	then lower-case($knowledgebase//feature[@id=$feature_id]/@informational)
	else "false"
};

declare function local:isFeatureInformational ($feature_id as xs:string)
as xs:boolean
{
	lower-case($knowledgebase//feature[@id=$feature_id]/@informational) = "true"
};

declare function local:generateSummaryItemForObjectType($preliminary_report, $object_type)
{
	let $description := $object_type
	let $number := sum($preliminary_plsql_report/report/statistics/item[@id = concat($object_type, "Count")]/result)
	let $number_incompatible := count(distinct-values($preliminary_plsql_report/report/features/feature[not(local:isFeatureInformational(@id))]/instance[upper-case(./object/data(@type)) = upper-case($object_type)]/object-id))
	let $percent_incompatible := meet:percent($number_incompatible, $number)
	return
		<item>
			<object_type>{ $description }</object_type>
			<number>{ $number }</number>
			<number_incompatible>{ $number_incompatible }</number_incompatible>
			<percent_incompatible>{ $percent_incompatible }</percent_incompatible>
		</item>
};

declare function local:generateReportByFeatures ($preliminary_report)
{
	<features>
	{
		for $feature in $preliminary_report//features/feature
		let $type := $feature/data(@type)
		let $feature_id := $feature/data(@id)
		let $feature_name := local:getFeatureName($feature_id)
		let $count := count($feature/instance)
		let $effort := $knowledgebase//feature[@id=$feature_id]/effort
		(: only report the feature if it has more than 0 instances  :)
		where $feature//instance
		return 
				<feature informational="{ local:getFeatureInformational($feature_id) }" type="{ $type }">
				<name>{ $feature_name }</name>
				<count>{ $count }</count>
				{ $effort }
				<description>{ local:getFeatureDescription($feature_id) }</description>
				<solution>{ local:getFeatureSolution($feature_id) }</solution>
				<instances>
				{
					for $instance in $feature/instance
					return
						<instance>
							{ $instance/lineno }
							{
								if (meetjava:getIsInternalExternal() = "internal")
								then $instance/context
								else ""
							}
							{ $instance/object }
						</instance>
				}
				</instances>
			</feature>
	}
	</features>
};


<report>

	<sources>
		<source>
			<type>File</type>
			<name>{ $preliminary_plsql_report//statistics/item[@id = 'FILE_PATH']/result/text() }</name>
		</source>
	</sources>

	<executeinfo>
		<datetime>
			{ current-dateTime() }
		</datetime>
		<internalexternal>
			{ meetjava:getIsInternalExternal() }
		</internalexternal>
		<version>
			<query>
				{ meetjava:getConfig('QUERY_VERSION_NAME') }
			</query>
			<kb>
				{ meetjava:getConfig('SYBASE_KB_VERSION') }
			</kb>
			<xcompiler>
				{ meetjava:getConfig('ANALYZER_VERSION') }
			</xcompiler>
			<meet>
				{ meetjava:getConfig('MEET_VERSION') }
			</meet>
		</version>
		<builddate>
			{ meetjava:getConfig('MEET_BUILD_DATE') }
		</builddate>
		<mode>Sybase</mode>
	</executeinfo>

	<summary>
		{ local:generateSummaryItemForObjectType($preliminary_plsql_report, 'Table') }
		{ local:generateSummaryItemForObjectType($preliminary_plsql_report, 'View') }
		{ local:generateSummaryItemForObjectType($preliminary_plsql_report, 'Trigger') }
		{ local:generateSummaryItemForObjectType($preliminary_plsql_report, 'Procedure') }
		{ local:generateSummaryItemForObjectType($preliminary_plsql_report, 'Rule') }
		{ local:generateSummaryItemForObjectType($preliminary_plsql_report, 'Default') }
		{ local:generateSummaryItemForObjectType($preliminary_plsql_report, 'Schema') }
		{ local:generateSummaryItemForObjectType($preliminary_plsql_report, 'Database') }
		{ local:generateSummaryItemForObjectType($preliminary_plsql_report, 'Index') }
		<item>
			<object_type>Total Objects</object_type>
			<number>{ sum($preliminary_plsql_report/report/statistics/item[ends-with(@id,"Count") and not(ends-with(@id,"StatementCount"))]/result) }</number>
			<number_incompatible>{ count(distinct-values($preliminary_plsql_report/report/features/feature[not(local:isFeatureInformational(@id))]/instance[exists(index-of(("TABLE", "VIEW", "TRIGGER", "PROCEDURE", "RULE", "DEFAULT", "SCHEMA", "DATABASE", "INDEX"), upper-case(./object/data(@type))))]/object-id)) }</number_incompatible>
			<percent_incompatible>{ meet:percent-decimalplaces(count(distinct-values($preliminary_plsql_report/report/features/feature[not(local:isFeatureInformational(@id))]/instance[exists(index-of(("TABLE", "VIEW", "TRIGGER", "PROCEDURE", "RULE", "DEFAULT", "SCHEMA", "DATABASE", "INDEX"), upper-case(./object/data(@type))))]/object-id)), sum($preliminary_plsql_report/report/statistics/item[ends-with(@id,"Count") and not(ends-with(@id,"StatementCount"))]/result), 1) }</percent_incompatible>
		</item>
		<item>
			<object_type>Statements</object_type>
			<number>{ $preliminary_plsql_report/report/statistics/item[@id="StatementCount"]/result/text() }</number>
			<number_incompatible>{ count(distinct-values($preliminary_plsql_report/report/features/feature[not(local:isFeatureInformational(@id))]/instance/statement-id)) }</number_incompatible>
			<percent_incompatible>{ meet:percent-decimalplaces(count(distinct-values($preliminary_plsql_report/report/features/feature[not(local:isFeatureInformational(@id))]/instance/statement-id)), $preliminary_plsql_report/report/statistics/item[@id="StatementCount"]/result/text(), 1) }</percent_incompatible>
		</item>
	</summary>

	<plsql>
		{ $preliminary_plsql_report//statistics }
	
		{ local:generateReportByFeatures($preliminary_plsql_report) }
	</plsql>


	{ $preliminary_plsql_report//unrecognized_grammars }

</report>
